@@ -12,7 +12,7 @@ char *lenient_strcat(char *dest, const char *src);
 char *lenient_strncat(char *dest, const char *src, size_t len);
 
 // strcmp that also handles null pointers.
-int lenient_strcmp(char *a, char *b);
+int lenient_strcmp(const char *a, const char *b);
 
 // Simply split a string with delims, free with `list_free_items_and_destroy`
 list_t *split_string(const char *str, const char *delims);
