@@ -112,6 +112,7 @@ sway_cmd cmd_border;
 sway_cmd cmd_client_noop;
 sway_cmd cmd_client_focused;
 sway_cmd cmd_client_focused_inactive;
+sway_cmd cmd_client_focused_tab_title;
 sway_cmd cmd_client_unfocused;
 sway_cmd cmd_client_urgent;
 sway_cmd cmd_client_placeholder;
@@ -282,7 +283,9 @@ sway_cmd output_cmd_dpms;
 sway_cmd output_cmd_enable;
 sway_cmd output_cmd_max_render_time;
 sway_cmd output_cmd_mode;
+sway_cmd output_cmd_modeline;
 sway_cmd output_cmd_position;
+sway_cmd output_cmd_render_bit_depth;
 sway_cmd output_cmd_scale;
 sway_cmd output_cmd_scale_filter;
 sway_cmd output_cmd_subpixel;
