@@ -5,9 +5,10 @@
 #include <string.h>
 #include <time.h>
 #include <wlr/interfaces/wlr_switch.h>
-#include <wlr/types/wlr_box.h>
 #include <wlr/types/wlr_tablet_tool.h>
+#include <wlr/util/box.h>
 #include <xkbcommon/xkbcommon.h>
+#include <xf86drmMode.h>
 #include "../include/config.h"
 #include "list.h"
 #include "swaynag.h"
@@ -246,6 +247,12 @@ enum scale_filter_mode {
 	SCALE_FILTER_SMART,
 };
 
+enum render_bit_depth {
+	RENDER_BIT_DEPTH_DEFAULT, // the default is currently 8
+	RENDER_BIT_DEPTH_8,
+	RENDER_BIT_DEPTH_10,
+};
+
 /**
  * Size and position configuration for a particular output.
  *
@@ -257,6 +264,7 @@ struct output_config {
 	int width, height;
 	float refresh_rate;
 	int custom_mode;
+	drmModeModeInfo drm_mode;
 	int x, y;
 	float scale;
 	enum scale_filter_mode scale_filter;
@@ -264,6 +272,7 @@ struct output_config {
 	enum wl_output_subpixel subpixel;
 	int max_render_time; // In milliseconds
 	int adaptive_sync;
+	enum render_bit_depth render_bit_depth;
 
 	char *background;
 	char *background_option;
@@ -281,6 +290,12 @@ struct side_gaps {
 	int left;
 };
 
+enum smart_gaps_mode {
+	SMART_GAPS_OFF,
+	SMART_GAPS_ON,
+	SMART_GAPS_INVERSE_OUTER,
+};
+
 /**
  * Stores configuration for a workspace, regardless of whether the workspace
  * exists.
@@ -292,6 +307,12 @@ struct workspace_config {
 	struct side_gaps gaps_outer;
 };
 
+enum pango_markup_config {
+	PANGO_MARKUP_DISABLED = false,
+	PANGO_MARKUP_ENABLED = true,
+	PANGO_MARKUP_DEFAULT // The default is font dependent ("pango:" prefix)
+};
+
 struct bar_config {
 	char *swaybar_command;
 	struct wl_client *client;
@@ -323,7 +344,7 @@ struct bar_config {
 	char *position;
 	list_t *bindings;
 	char *status_command;
-	bool pango_markup;
+	enum pango_markup_config pango_markup;
 	char *font;
 	int height; // -1 not defined
 	bool workspace_buttons;
@@ -410,14 +431,6 @@ enum sway_popup_during_fullscreen {
 	POPUP_LEAVE,
 };
 
-enum command_context {
-	CONTEXT_CONFIG = 1 << 0,
-	CONTEXT_BINDING = 1 << 1,
-	CONTEXT_IPC = 1 << 2,
-	CONTEXT_CRITERIA = 1 << 3,
-	CONTEXT_ALL = 0xFFFFFFFF,
-};
-
 enum focus_follows_mouse_mode {
 	FOLLOWS_NO,
 	FOLLOWS_YES,
@@ -480,8 +493,8 @@ struct sway_config {
 	enum sway_container_layout default_orientation;
 	enum sway_container_layout default_layout;
 	char *font;
-	size_t font_height;
-	size_t font_baseline;
+	int font_height;
+	int font_baseline;
 	bool pango_markup;
 	int titlebar_border_thickness;
 	int titlebar_h_padding;
@@ -512,7 +525,7 @@ struct sway_config {
 	bool tiling_drag;
 	int tiling_drag_threshold;
 
-	bool smart_gaps;
+	enum smart_gaps_mode smart_gaps;
 	int gaps_inner;
 	struct side_gaps gaps_outer;
 
@@ -535,12 +548,15 @@ struct sway_config {
 	struct {
 		struct border_colors focused;
 		struct border_colors focused_inactive;
+		struct border_colors focused_tab_title;
 		struct border_colors unfocused;
 		struct border_colors urgent;
 		struct border_colors placeholder;
 		float background[4];
 	} border_colors;
 
+	bool has_focused_tab_title;
+
 	// floating view
 	int32_t floating_maximum_width;
 	int32_t floating_maximum_height;
@@ -690,14 +706,13 @@ void free_bar_binding(struct bar_binding *binding);
 void free_workspace_config(struct workspace_config *wsc);
 
 /**
- * Updates the value of config->font_height based on the max title height
- * reported by each container. If recalculate is true, the containers will
- * recalculate their heights before reporting.
- *
+ * Updates the value of config->font_height based on the metrics for title's
+ * font as reported by pango.
+ * 
  * If the height has changed, all containers will be rearranged to take on the
  * new size.
  */
-void config_update_font_height(bool recalculate);
+void config_update_font_height(void);
 
 /**
  * Convert bindsym into bindcode using the first configured layout.
