@@ -3,7 +3,6 @@
 #include <time.h>
 #include <unistd.h>
 #include <wayland-server-core.h>
-#include <wlr/types/wlr_box.h>
 #include <wlr/types/wlr_output.h>
 #include "config.h"
 #include "sway/tree/node.h"
@@ -49,7 +48,7 @@ struct sway_output {
 	struct wl_listener damage_frame;
 
 	struct {
-		struct wl_signal destroy;
+		struct wl_signal disable;
 	} events;
 
 	struct timespec last_presentation;
@@ -72,8 +71,8 @@ struct sway_output *output_get_in_direction(struct sway_output *reference,
 void output_add_workspace(struct sway_output *output,
 		struct sway_workspace *workspace);
 
-typedef void (*sway_surface_iterator_func_t)(struct sway_output *output, struct sway_view *view,
-	struct wlr_surface *surface, struct wlr_box *box, float rotation,
+typedef void (*sway_surface_iterator_func_t)(struct sway_output *output,
+	struct sway_view *view, struct wlr_surface *surface, struct wlr_box *box,
 	void *user_data);
 
 void output_damage_whole(struct sway_output *output);
