@@ -4,11 +4,13 @@
 #include <wayland-server-core.h>
 #include <wlr/backend.h>
 #include <wlr/backend/session.h>
+#include <wlr/render/allocator.h>
 #include <wlr/render/wlr_renderer.h>
 #include <wlr/types/wlr_compositor.h>
 #include <wlr/types/wlr_data_device.h>
 #include <wlr/types/wlr_input_method_v2.h>
 #include <wlr/types/wlr_foreign_toplevel_management_v1.h>
+#include <wlr/types/wlr_drm_lease_v1.h>
 #include <wlr/types/wlr_layer_shell_v1.h>
 #include <wlr/types/wlr_output_management_v1.h>
 #include <wlr/types/wlr_output_power_management_v1.h>
@@ -31,13 +33,16 @@ struct sway_server {
 	const char *socket;
 
 	struct wlr_backend *backend;
-	struct wlr_backend *noop_backend;
 	// secondary headless backend used for creating virtual outputs on-the-fly
 	struct wlr_backend *headless_backend;
+	struct wlr_renderer *renderer;
+	struct wlr_allocator *allocator;
 
 	struct wlr_compositor *compositor;
 	struct wl_listener compositor_new_surface;
 
+	struct wlr_linux_dmabuf_v1 *linux_dmabuf_v1;
+
 	struct wlr_data_device_manager *data_device_manager;
 
 	struct sway_input_manager *input;
@@ -72,6 +77,9 @@ struct sway_server {
 	struct wl_listener xdg_decoration;
 	struct wl_list xdg_decorations; // sway_xdg_decoration::link
 
+	struct wlr_drm_lease_v1_manager *drm_lease_manager;
+	struct wl_listener drm_lease_request;
+
 	struct wlr_presentation *presentation;
 
 	struct wlr_pointer_constraints_v1 *pointer_constraints;
@@ -87,6 +95,9 @@ struct sway_server {
 	struct wlr_text_input_manager_v3 *text_input;
 	struct wlr_foreign_toplevel_manager_v1 *foreign_toplevel_manager;
 
+	struct wlr_xdg_activation_v1 *xdg_activation_v1;
+	struct wl_listener xdg_activation_v1_request_activate;
+
 	// The timeout for transactions, after which a transaction is applied
 	// regardless of readiness.
 	size_t txn_timeout_ms;
@@ -112,6 +123,7 @@ struct sway_debug {
 	bool noatomic;         // Ignore atomic layout updates
 	bool txn_timings;      // Log verbose messages about transactions
 	bool txn_wait;         // Always wait for the timeout before applying
+	bool noscanout;        // Disable direct scan-out
 
 	enum {
 		DAMAGE_DEFAULT,    // Default behaviour
@@ -129,6 +141,8 @@ void server_fini(struct sway_server *server);
 bool server_start(struct sway_server *server);
 void server_run(struct sway_server *server);
 
+void restore_nofile_limit(void);
+
 void handle_compositor_new_surface(struct wl_listener *listener, void *data);
 void handle_new_output(struct wl_listener *listener, void *data);
 
@@ -141,5 +155,7 @@ void handle_xwayland_surface(struct wl_listener *listener, void *data);
 void handle_server_decoration(struct wl_listener *listener, void *data);
 void handle_xdg_decoration(struct wl_listener *listener, void *data);
 void handle_pointer_constraint(struct wl_listener *listener, void *data);
+void xdg_activation_v1_handle_request_activate(struct wl_listener *listener,
+	void *data);
 
 #endif
