@@ -2,7 +2,6 @@
 #define _SWAY_CONTAINER_H
 #include <stdint.h>
 #include <sys/types.h>
-#include <wlr/types/wlr_box.h>
 #include <wlr/types/wlr_surface.h>
 #include "list.h"
 #include "sway/tree/node.h"
@@ -118,14 +117,14 @@ struct sway_container {
 
 	struct wlr_texture *title_focused;
 	struct wlr_texture *title_focused_inactive;
+	struct wlr_texture *title_focused_tab_title;
 	struct wlr_texture *title_unfocused;
 	struct wlr_texture *title_urgent;
-	size_t title_height;
-	size_t title_baseline;
 
 	list_t *marks; // char *
 	struct wlr_texture *marks_focused;
 	struct wlr_texture *marks_focused_inactive;
+	struct wlr_texture *marks_focused_tab_title;
 	struct wlr_texture *marks_unfocused;
 	struct wlr_texture *marks_urgent;
 
@@ -184,11 +183,6 @@ struct sway_container *container_flatten(struct sway_container *container);
 
 void container_update_title_textures(struct sway_container *container);
 
-/**
- * Calculate the container's title_height property.
- */
-void container_calculate_title_height(struct sway_container *container);
-
 size_t container_build_representation(enum sway_container_layout layout,
 		list_t *children, char *buffer);
 
