@@ -41,6 +41,7 @@ struct swaybar {
 	int ipc_socketfd;
 
 	struct wl_list outputs; // swaybar_output::link
+	struct wl_list unused_outputs; // swaybar_output::link
 	struct wl_list seats; // swaybar_seat::link
 
 #if HAVE_TRAY
@@ -109,4 +110,8 @@ void set_bar_dirty(struct swaybar *bar);
 bool determine_bar_visibility(struct swaybar *bar, bool moving_layer);
 void free_workspaces(struct wl_list *list);
 
+void status_in(int fd, short mask, void *data);
+
+void destroy_layer_surface(struct swaybar_output *output);
+
 #endif
