@@ -22,6 +22,6 @@ struct cmd_results *cmd_font(int argc, char **argv) {
 	}
 
 	free(font);
-	config_update_font_height(true);
+	config_update_font_height();
 	return cmd_results_new(CMD_SUCCESS, NULL);
 }
