@@ -1,7 +1,6 @@
 #define _POSIX_C_SOURCE 200809L
 #include <stdbool.h>
 #include <string.h>
-#include <strings.h>
 #include "sway/commands.h"
 #include "sway/config.h"
 #include "sway/ipc-server.h"
@@ -10,11 +9,14 @@
 #include "stringop.h"
 
 // Must be in order for the bsearch
-static struct cmd_handler mode_handlers[] = {
+static const struct cmd_handler mode_handlers[] = {
 	{ "bindcode", cmd_bindcode },
 	{ "bindswitch", cmd_bindswitch },
 	{ "bindsym", cmd_bindsym },
 	{ "set", cmd_set },
+	{ "unbindcode", cmd_unbindcode },
+	{ "unbindswitch", cmd_unbindswitch },
+	{ "unbindsym", cmd_unbindsym },
 };
 
 struct cmd_results *cmd_mode(int argc, char **argv) {
@@ -23,10 +25,6 @@ struct cmd_results *cmd_mode(int argc, char **argv) {
 		return error;
 	}
 
-	if (argc > 1 && !config->reading) {
-		return cmd_results_new(CMD_FAILURE, "Can only be used in config file");
-	}
-
 	bool pango = strcmp(*argv, "--pango_markup") == 0;
 	if (pango) {
 		argc--; argv++;
@@ -35,13 +33,17 @@ struct cmd_results *cmd_mode(int argc, char **argv) {
 		}
 	}
 
+	if (config->reading && argc == 1) {
+		return cmd_results_new(CMD_DEFER, NULL);
+	}
+
 	char *mode_name = *argv;
 	strip_quotes(mode_name);
 	struct sway_mode *mode = NULL;
 	// Find mode
 	for (int i = 0; i < config->modes->length; ++i) {
 		struct sway_mode *test = config->modes->items[i];
-		if (strcasecmp(test->name, mode_name) == 0) {
+		if (strcmp(test->name, mode_name) == 0) {
 			mode = test;
 			break;
 		}
@@ -64,14 +66,13 @@ struct cmd_results *cmd_mode(int argc, char **argv) {
 		error = cmd_results_new(CMD_INVALID, "Unknown mode `%s'", mode_name);
 		return error;
 	}
-	if ((config->reading && argc > 1) || (!config->reading && argc == 1)) {
-		sway_log(SWAY_DEBUG, "Switching to mode `%s' (pango=%d)",
-				mode->name, mode->pango);
-	}
 	// Set current mode
+	struct sway_mode *stored_mode = config->current_mode;
 	config->current_mode = mode;
 	if (argc == 1) {
 		// trigger IPC mode event
+		sway_log(SWAY_DEBUG, "Switching to mode `%s' (pango=%d)",
+				mode->name, mode->pango);
 		ipc_event_mode(config->current_mode->name,
 				config->current_mode->pango);
 		return cmd_results_new(CMD_SUCCESS, NULL);
@@ -80,7 +81,7 @@ struct cmd_results *cmd_mode(int argc, char **argv) {
 	// Create binding
 	struct cmd_results *result = config_subcommand(argv + 1, argc - 1,
 			mode_handlers, sizeof(mode_handlers));
-	config->current_mode = config->modes->items[0];
+	config->current_mode = stored_mode;
 
 	return result;
 }
