@@ -2,7 +2,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <wayland-server-core.h>
-#include <wlr/types/wlr_box.h>
 #include <wlr/types/wlr_layer_shell_v1.h>
 #include <wlr/types/wlr_output_damage.h>
 #include <wlr/types/wlr_output.h>
@@ -163,9 +162,8 @@ static void arrange_layer(struct sway_output *output, struct wl_list *list,
 		} else if ((state->anchor & ZWLR_LAYER_SURFACE_V1_ANCHOR_BOTTOM)) {
 			box.y -= state->margin.bottom;
 		}
-		if (box.width < 0 || box.height < 0) {
-			// TODO: Bubble up a protocol error?
-			wlr_layer_surface_v1_close(layer);
+		if (!sway_assert(box.width >= 0 && box.height >= 0,
+				"Expected layer surface to have positive size")) {
 			continue;
 		}
 		// Apply
@@ -199,7 +197,7 @@ void arrange_layers(struct sway_output *output) {
 		arrange_output(output);
 	}
 
-	// Arrange non-exlusive surfaces from top->bottom
+	// Arrange non-exclusive surfaces from top->bottom
 	arrange_layer(output, &output->layers[ZWLR_LAYER_SHELL_V1_LAYER_OVERLAY],
 			&usable_area, false);
 	arrange_layer(output, &output->layers[ZWLR_LAYER_SHELL_V1_LAYER_TOP],
@@ -285,7 +283,7 @@ static void handle_output_destroy(struct wl_listener *listener, void *data) {
 	}
 
 	sway_layer->layer_surface->output = NULL;
-	wlr_layer_surface_v1_close(sway_layer->layer_surface);
+	wlr_layer_surface_v1_destroy(sway_layer->layer_surface);
 }
 
 static void handle_surface_commit(struct wl_listener *listener, void *data) {
@@ -298,21 +296,30 @@ static void handle_surface_commit(struct wl_listener *listener, void *data) {
 	}
 
 	struct sway_output *output = wlr_output->data;
-	struct wlr_box old_geo = layer->geo;
-	arrange_layers(output);
+	struct wlr_box old_extent = layer->extent;
 
-	bool geo_changed =
-		memcmp(&old_geo, &layer->geo, sizeof(struct wlr_box)) != 0;
-	bool layer_changed = layer->layer != layer_surface->current.layer;
-	if (layer_changed) {
-		wl_list_remove(&layer->link);
-		wl_list_insert(&output->layers[layer_surface->current.layer],
-			&layer->link);
-		layer->layer = layer_surface->current.layer;
+	bool layer_changed = false;
+	if (layer_surface->current.committed != 0
+			|| layer->mapped != layer_surface->mapped) {
+		layer->mapped = layer_surface->mapped;
+		layer_changed = layer->layer != layer_surface->current.layer;
+		if (layer_changed) {
+			wl_list_remove(&layer->link);
+			wl_list_insert(&output->layers[layer_surface->current.layer],
+				&layer->link);
+			layer->layer = layer_surface->current.layer;
+		}
+		arrange_layers(output);
 	}
-	if (geo_changed || layer_changed) {
-		output_damage_surface(output, old_geo.x, old_geo.y,
-			layer_surface->surface, true);
+
+	wlr_surface_get_extends(layer_surface->surface, &layer->extent);
+	layer->extent.x += layer->geo.x;
+	layer->extent.y += layer->geo.y;
+
+	bool extent_changed =
+		memcmp(&old_extent, &layer->extent, sizeof(struct wlr_box)) != 0;
+	if (extent_changed || layer_changed) {
+		output_damage_box(output, &old_extent);
 		output_damage_surface(output, layer->geo.x, layer->geo.y,
 			layer_surface->surface, true);
 	} else {
@@ -345,6 +352,8 @@ static void unmap(struct sway_layer_surface *sway_layer) {
 		sway_layer->layer_surface->surface, true);
 }
 
+static void layer_subsurface_destroy(struct sway_layer_subsurface *subsurface);
+
 static void handle_destroy(struct wl_listener *listener, void *data) {
 	struct sway_layer_surface *sway_layer =
 		wl_container_of(listener, sway_layer, destroy);
@@ -353,6 +362,12 @@ static void handle_destroy(struct wl_listener *listener, void *data) {
 	if (sway_layer->layer_surface->mapped) {
 		unmap(sway_layer);
 	}
+
+	struct sway_layer_subsurface *subsurface, *subsurface_tmp;
+	wl_list_for_each_safe(subsurface, subsurface_tmp, &sway_layer->subsurfaces, link) {
+		layer_subsurface_destroy(subsurface);
+	}
+
 	wl_list_remove(&sway_layer->link);
 	wl_list_remove(&sway_layer->destroy.link);
 	wl_list_remove(&sway_layer->map.link);
@@ -421,11 +436,8 @@ static void subsurface_handle_commit(struct wl_listener *listener, void *data) {
 	subsurface_damage(subsurface, false);
 }
 
-static void subsurface_handle_destroy(struct wl_listener *listener,
-		void *data) {
-	struct sway_layer_subsurface *subsurface =
-			wl_container_of(listener, subsurface, destroy);
-
+static void layer_subsurface_destroy(struct sway_layer_subsurface *subsurface) {
+	wl_list_remove(&subsurface->link);
 	wl_list_remove(&subsurface->map.link);
 	wl_list_remove(&subsurface->unmap.link);
 	wl_list_remove(&subsurface->destroy.link);
@@ -433,6 +445,13 @@ static void subsurface_handle_destroy(struct wl_listener *listener,
 	free(subsurface);
 }
 
+static void subsurface_handle_destroy(struct wl_listener *listener,
+		void *data) {
+	struct sway_layer_subsurface *subsurface =
+			wl_container_of(listener, subsurface, destroy);
+	layer_subsurface_destroy(subsurface);
+}
+
 static struct sway_layer_subsurface *create_subsurface(
 		struct wlr_subsurface *wlr_subsurface,
 		struct sway_layer_surface *layer_surface) {
@@ -444,6 +463,7 @@ static struct sway_layer_subsurface *create_subsurface(
 
 	subsurface->wlr_subsurface = wlr_subsurface;
 	subsurface->layer_surface = layer_surface;
+	wl_list_insert(&layer_surface->subsurfaces, &subsurface->link);
 
 	subsurface->map.notify = subsurface_handle_map;
 	wl_signal_add(&wlr_subsurface->events.map, &subsurface->map);
@@ -476,8 +496,8 @@ static struct sway_layer_surface *popup_get_layer(
 static void popup_damage(struct sway_layer_popup *layer_popup, bool whole) {
 	struct wlr_xdg_popup *popup = layer_popup->wlr_popup;
 	struct wlr_surface *surface = popup->base->surface;
-	int popup_sx = popup->geometry.x - popup->base->geometry.x;
-	int popup_sy = popup->geometry.y - popup->base->geometry.y;
+	int popup_sx = popup->geometry.x - popup->base->current.geometry.x;
+	int popup_sy = popup->geometry.y - popup->base->current.geometry.y;
 	int ox = popup_sx, oy = popup_sy;
 	struct sway_layer_surface *layer;
 	while (true) {
@@ -598,14 +618,14 @@ void handle_layer_shell_surface(struct wl_listener *listener, void *data) {
 	sway_log(SWAY_DEBUG, "new layer surface: namespace %s layer %d anchor %" PRIu32
 			" size %" PRIu32 "x%" PRIu32 " margin %" PRIu32 ",%" PRIu32 ",%" PRIu32 ",%" PRIu32 ",",
 		layer_surface->namespace,
-		layer_surface->client_pending.layer,
-		layer_surface->client_pending.anchor,
-		layer_surface->client_pending.desired_width,
-		layer_surface->client_pending.desired_height,
-		layer_surface->client_pending.margin.top,
-		layer_surface->client_pending.margin.right,
-		layer_surface->client_pending.margin.bottom,
-		layer_surface->client_pending.margin.left);
+		layer_surface->pending.layer,
+		layer_surface->pending.anchor,
+		layer_surface->pending.desired_width,
+		layer_surface->pending.desired_height,
+		layer_surface->pending.margin.top,
+		layer_surface->pending.margin.right,
+		layer_surface->pending.margin.bottom,
+		layer_surface->pending.margin.left);
 
 	if (!layer_surface->output) {
 		// Assign last active output
@@ -617,12 +637,12 @@ void handle_layer_shell_surface(struct wl_listener *listener, void *data) {
 				output = ws->output;
 			}
 		}
-		if (!output || output == root->noop_output) {
+		if (!output || output == root->fallback_output) {
 			if (!root->outputs->length) {
 				sway_log(SWAY_ERROR,
 						"no output to auto-assign layer surface '%s' to",
 						layer_surface->namespace);
-				wlr_layer_surface_v1_close(layer_surface);
+				wlr_layer_surface_v1_destroy(layer_surface);
 				return;
 			}
 			output = root->outputs->items[0];
@@ -636,6 +656,8 @@ void handle_layer_shell_surface(struct wl_listener *listener, void *data) {
 		return;
 	}
 
+	wl_list_init(&sway_layer->subsurfaces);
+
 	sway_layer->surface_commit.notify = handle_surface_commit;
 	wl_signal_add(&layer_surface->surface->events.commit,
 		&sway_layer->surface_commit);
@@ -657,15 +679,15 @@ void handle_layer_shell_surface(struct wl_listener *listener, void *data) {
 
 	struct sway_output *output = layer_surface->output->data;
 	sway_layer->output_destroy.notify = handle_output_destroy;
-	wl_signal_add(&output->events.destroy, &sway_layer->output_destroy);
+	wl_signal_add(&output->events.disable, &sway_layer->output_destroy);
 
-	wl_list_insert(&output->layers[layer_surface->client_pending.layer],
+	wl_list_insert(&output->layers[layer_surface->pending.layer],
 			&sway_layer->link);
 
-	// Temporarily set the layer's current state to client_pending
+	// Temporarily set the layer's current state to pending
 	// So that we can easily arrange it
 	struct wlr_layer_surface_v1_state old_state = layer_surface->current;
-	layer_surface->current = layer_surface->client_pending;
+	layer_surface->current = layer_surface->pending;
 	arrange_layers(output);
 	layer_surface->current = old_state;
 }
