@@ -4,9 +4,11 @@
 #include <strings.h>
 #include <time.h>
 #include <wayland-server-core.h>
+#include <wlr/backend/drm.h>
+#include <wlr/backend/headless.h>
 #include <wlr/render/wlr_renderer.h>
-#include <wlr/types/wlr_box.h>
 #include <wlr/types/wlr_buffer.h>
+#include <wlr/types/wlr_drm_lease_v1.h>
 #include <wlr/types/wlr_matrix.h>
 #include <wlr/types/wlr_output_damage.h>
 #include <wlr/types/wlr_output_layout.h>
@@ -56,26 +58,6 @@ struct sway_output *all_output_by_name_or_id(const char *name_or_id) {
 	return NULL;
 }
 
-/**
- * Rotate a child's position relative to a parent. The parent size is (pw, ph),
- * the child position is (*sx, *sy) and its size is (sw, sh).
- */
-static void rotate_child_position(double *sx, double *sy, double sw, double sh,
-		double pw, double ph, float rotation) {
-	if (rotation == 0.0f) {
-		return;
-	}
-
-	// Coordinates relative to the center of the subsurface
-	double ox = *sx - pw/2 + sw/2,
-		oy = *sy - ph/2 + sh/2;
-	// Rotated coordinates
-	double rx = cos(-rotation)*ox - sin(-rotation)*oy,
-		ry = cos(-rotation)*oy + sin(-rotation)*ox;
-	*sx = rx + pw/2 - sw/2;
-	*sy = ry + ph/2 - sh/2;
-}
-
 struct surface_iterator_data {
 	sway_surface_iterator_func_t user_iterator;
 	void *user_data;
@@ -84,7 +66,6 @@ struct surface_iterator_data {
 	struct sway_view *view;
 	double ox, oy;
 	int width, height;
-	float rotation;
 };
 
 static bool get_surface_box(struct surface_iterator_data *data,
@@ -99,14 +80,9 @@ static bool get_surface_box(struct surface_iterator_data *data,
 	int sw = surface->current.width;
 	int sh = surface->current.height;
 
-	double _sx = sx;
-	double _sy = sy;
-	rotate_child_position(&_sx, &_sy, sw, sh, data->width, data->height,
-		data->rotation);
-
 	struct wlr_box box = {
-		.x = floor(data->ox + _sx),
-		.y = floor(data->oy + _sy),
+		.x = floor(data->ox + sx),
+		.y = floor(data->oy + sy),
 		.width = sw,
 		.height = sh,
 	};
@@ -114,16 +90,13 @@ static bool get_surface_box(struct surface_iterator_data *data,
 		memcpy(surface_box, &box, sizeof(struct wlr_box));
 	}
 
-	struct wlr_box rotated_box;
-	wlr_box_rotated_bounds(&rotated_box, &box, data->rotation);
-
 	struct wlr_box output_box = {
 		.width = output->width,
 		.height = output->height,
 	};
 
 	struct wlr_box intersection;
-	return wlr_box_intersection(&intersection, &output_box, &rotated_box);
+	return wlr_box_intersection(&intersection, &output_box, &box);
 }
 
 static void output_for_each_surface_iterator(struct wlr_surface *surface,
@@ -136,7 +109,7 @@ static void output_for_each_surface_iterator(struct wlr_surface *surface,
 		return;
 	}
 
-	data->user_iterator(data->output, data->view, surface, &box, data->rotation,
+	data->user_iterator(data->output, data->view, surface, &box,
 		data->user_data);
 }
 
@@ -152,7 +125,6 @@ void output_surface_for_each_surface(struct sway_output *output,
 		.oy = oy,
 		.width = surface->current.width,
 		.height = surface->current.height,
-		.rotation = 0,
 	};
 
 	wlr_surface_for_each_surface(surface,
@@ -173,7 +145,6 @@ void output_view_for_each_surface(struct sway_output *output,
 			- view->geometry.y,
 		.width = view->container->current.content_width,
 		.height = view->container->current.content_height,
-		.rotation = 0, // TODO
 	};
 
 	view_for_each_surface(view, output_for_each_surface_iterator, &data);
@@ -193,7 +164,6 @@ void output_view_for_each_popup_surface(struct sway_output *output,
 			- view->geometry.y,
 		.width = view->container->current.content_width,
 		.height = view->container->current.content_height,
-		.rotation = 0, // TODO
 	};
 
 	view_for_each_popup_surface(view, output_for_each_surface_iterator, &data);
@@ -216,7 +186,6 @@ void output_layer_for_each_surface(struct sway_output *output,
 			.oy = layer_surface->geo.y,
 			.width = surface->current.width,
 			.height = surface->current.height,
-			.rotation = 0,
 		};
 		wlr_layer_surface_v1_for_each_surface(wlr_layer_surface_v1,
 			output_for_each_surface_iterator, &data);
@@ -254,7 +223,6 @@ void output_layer_for_each_popup_surface(struct sway_output *output,
 			.oy = layer_surface->geo.y,
 			.width = surface->current.width,
 			.height = surface->current.height,
-			.rotation = 0,
 		};
 		wlr_layer_surface_v1_for_each_popup_surface(wlr_layer_surface_v1,
 			output_for_each_surface_iterator, &data);
@@ -426,9 +394,9 @@ struct send_frame_done_data {
 	int msec_until_refresh;
 };
 
-static void send_frame_done_iterator(struct sway_output *output, struct sway_view *view,
-		struct wlr_surface *surface, struct wlr_box *box, float rotation,
-		void *user_data) {
+static void send_frame_done_iterator(struct sway_output *output,
+		struct sway_view *view, struct wlr_surface *surface,
+		struct wlr_box *box, void *user_data) {
 	int view_max_render_time = 0;
 	if (view != NULL) {
 		view_max_render_time = view->max_render_time;
@@ -451,9 +419,9 @@ static void send_frame_done(struct sway_output *output, struct send_frame_done_d
 	output_for_each_surface(output, send_frame_done_iterator, data);
 }
 
-static void count_surface_iterator(struct sway_output *output, struct sway_view *view,
-		struct wlr_surface *surface, struct wlr_box *_box, float rotation,
-		void *data) {
+static void count_surface_iterator(struct sway_output *output,
+		struct sway_view *view, struct wlr_surface *surface,
+		struct wlr_box *box, void *data) {
 	size_t *n = data;
 	(*n)++;
 }
@@ -540,7 +508,7 @@ static int output_repaint_timer_handler(void *data) {
 		fullscreen_con = workspace->current.fullscreen;
 	}
 
-	if (fullscreen_con && fullscreen_con->view) {
+	if (fullscreen_con && fullscreen_con->view && !debug.noscanout) {
 		// Try to scan-out the fullscreen view
 		static bool last_scanned_out = false;
 		bool scanned_out =
@@ -657,38 +625,30 @@ void output_damage_whole(struct sway_output *output) {
 	}
 }
 
-static void damage_surface_iterator(struct sway_output *output, struct sway_view *view,
-		struct wlr_surface *surface, struct wlr_box *_box, float rotation,
-		void *_data) {
+static void damage_surface_iterator(struct sway_output *output,
+		struct sway_view *view, struct wlr_surface *surface,
+		struct wlr_box *_box, void *_data) {
 	bool *data = _data;
 	bool whole = *data;
 
 	struct wlr_box box = *_box;
 	scale_box(&box, output->wlr_output->scale);
 
-	int center_x = box.x + box.width/2;
-	int center_y = box.y + box.height/2;
-
-	if (pixman_region32_not_empty(&surface->buffer_damage)) {
-		pixman_region32_t damage;
-		pixman_region32_init(&damage);
-		wlr_surface_get_effective_damage(surface, &damage);
-		wlr_region_scale(&damage, &damage, output->wlr_output->scale);
-		if (ceil(output->wlr_output->scale) > surface->current.scale) {
-			// When scaling up a surface, it'll become blurry so we need to
-			// expand the damage region
-			wlr_region_expand(&damage, &damage,
-				ceil(output->wlr_output->scale) - surface->current.scale);
-		}
-		pixman_region32_translate(&damage, box.x, box.y);
-		wlr_region_rotated_bounds(&damage, &damage, rotation,
-			center_x, center_y);
-		wlr_output_damage_add(output->damage, &damage);
-		pixman_region32_fini(&damage);
+	pixman_region32_t damage;
+	pixman_region32_init(&damage);
+	wlr_surface_get_effective_damage(surface, &damage);
+	wlr_region_scale(&damage, &damage, output->wlr_output->scale);
+	if (ceil(output->wlr_output->scale) > surface->current.scale) {
+		// When scaling up a surface, it'll become blurry so we need to
+		// expand the damage region
+		wlr_region_expand(&damage, &damage,
+			ceil(output->wlr_output->scale) - surface->current.scale);
 	}
+	pixman_region32_translate(&damage, box.x, box.y);
+	wlr_output_damage_add(output->damage, &damage);
+	pixman_region32_fini(&damage);
 
 	if (whole) {
-		wlr_box_rotated_bounds(&box, &box, rotation);
 		wlr_output_damage_add_box(output->damage, &box);
 	}
 
@@ -772,7 +732,7 @@ static void update_output_manager_config(struct sway_server *server) {
 
 	struct sway_output *output;
 	wl_list_for_each(output, &root->all_outputs, link) {
-		if (output == root->noop_output) {
+		if (output == root->fallback_output) {
 			continue;
 		}
 		struct wlr_output_configuration_head_v1 *config_head =
@@ -794,18 +754,22 @@ static void update_output_manager_config(struct sway_server *server) {
 static void handle_destroy(struct wl_listener *listener, void *data) {
 	struct sway_output *output = wl_container_of(listener, output, destroy);
 	struct sway_server *server = output->server;
-	wl_signal_emit(&output->events.destroy, output);
+	output_begin_destroy(output);
 
 	if (output->enabled) {
 		output_disable(output);
 	}
-	output_begin_destroy(output);
+
+	wl_list_remove(&output->link);
 
 	wl_list_remove(&output->destroy.link);
 	wl_list_remove(&output->commit.link);
 	wl_list_remove(&output->mode.link);
 	wl_list_remove(&output->present.link);
 
+	output->wlr_output->data = NULL;
+	output->wlr_output = NULL;
+
 	transaction_commit_dirty();
 
 	update_output_manager_config(server);
@@ -866,7 +830,7 @@ static void handle_present(struct wl_listener *listener, void *data) {
 	struct sway_output *output = wl_container_of(listener, output, present);
 	struct wlr_output_event_present *output_event = data;
 
-	if (!output->enabled) {
+	if (!output->enabled || !output_event->presented) {
 		return;
 	}
 
@@ -874,10 +838,39 @@ static void handle_present(struct wl_listener *listener, void *data) {
 	output->refresh_nsec = output_event->refresh;
 }
 
+static unsigned int last_headless_num = 0;
+
 void handle_new_output(struct wl_listener *listener, void *data) {
 	struct sway_server *server = wl_container_of(listener, server, new_output);
 	struct wlr_output *wlr_output = data;
-	sway_log(SWAY_DEBUG, "New output %p: %s", wlr_output, wlr_output->name);
+
+	if (wlr_output == root->fallback_output->wlr_output) {
+		return;
+	}
+
+	if (wlr_output_is_headless(wlr_output)) {
+		char name[64];
+		snprintf(name, sizeof(name), "HEADLESS-%u", ++last_headless_num);
+		wlr_output_set_name(wlr_output, name);
+	}
+
+	sway_log(SWAY_DEBUG, "New output %p: %s (non-desktop: %d)",
+			wlr_output, wlr_output->name, wlr_output->non_desktop);
+
+	if (wlr_output->non_desktop) {
+		sway_log(SWAY_DEBUG, "Not configuring non-desktop output");
+		if (server->drm_lease_manager) {
+			wlr_drm_lease_v1_manager_offer_output(server->drm_lease_manager,
+					wlr_output);
+		}
+		return;
+	}
+
+	if (!wlr_output_init_render(wlr_output, server->allocator,
+			server->renderer)) {
+		sway_log(SWAY_ERROR, "Failed to init output render");
+		return;
+	}
 
 	struct sway_output *output = output_create(wlr_output);
 	if (!output) {
