@@ -254,6 +254,7 @@ static void set_activated(struct sway_view *view, bool activated) {
 	}
 
 	wlr_xwayland_surface_activate(surface, activated);
+	wlr_xwayland_surface_restack(surface, NULL, XCB_STACK_MODE_ABOVE);
 }
 
 static void set_tiled(struct sway_view *view, bool tiled) {
@@ -435,6 +436,8 @@ static void handle_destroy(struct wl_listener *listener, void *data) {
 		wl_list_remove(&xwayland_view->commit.link);
 	}
 
+	xwayland_view->view.wlr_xwayland_surface = NULL;
+
 	wl_list_remove(&xwayland_view->destroy.link);
 	wl_list_remove(&xwayland_view->request_configure.link);
 	wl_list_remove(&xwayland_view->request_fullscreen.link);
