@@ -6,7 +6,6 @@
 #include <errno.h>
 #include <time.h>
 #include <strings.h>
-#include <wlr/types/wlr_box.h>
 #include <wlr/types/wlr_cursor.h>
 #include <wlr/types/wlr_idle.h>
 #include <wlr/types/wlr_pointer.h>
@@ -84,7 +83,28 @@ static struct wlr_surface *layer_surface_popup_at(struct sway_output *output,
 struct sway_node *node_at_coords(
 		struct sway_seat *seat, double lx, double ly,
 		struct wlr_surface **surface, double *sx, double *sy) {
-	// check for unmanaged views first
+	// find the output the cursor is on
+	struct wlr_output *wlr_output = wlr_output_layout_output_at(
+			root->output_layout, lx, ly);
+	if (wlr_output == NULL) {
+		return NULL;
+	}
+	struct sway_output *output = wlr_output->data;
+	if (!output || !output->enabled) {
+		// output is being destroyed or is being enabled
+		return NULL;
+	}
+	double ox = lx, oy = ly;
+	wlr_output_layout_output_coords(root->output_layout, wlr_output, &ox, &oy);
+
+	// layer surfaces on the overlay layer are rendered on top
+	if ((*surface = layer_surface_at(output,
+				&output->layers[ZWLR_LAYER_SHELL_V1_LAYER_OVERLAY],
+				ox, oy, sx, sy))) {
+		return NULL;
+	}
+
+	// check for unmanaged views
 #if HAVE_XWAYLAND
 	struct wl_list *unmanaged = &root->xwayland_unmanaged;
 	struct sway_xwayland_unmanaged *unmanaged_surface;
@@ -102,19 +122,6 @@ struct sway_node *node_at_coords(
 		}
 	}
 #endif
-	// find the output the cursor is on
-	struct wlr_output *wlr_output = wlr_output_layout_output_at(
-			root->output_layout, lx, ly);
-	if (wlr_output == NULL) {
-		return NULL;
-	}
-	struct sway_output *output = wlr_output->data;
-	if (!output || !output->enabled) {
-		// output is being destroyed or is being enabled
-		return NULL;
-	}
-	double ox = lx, oy = ly;
-	wlr_output_layout_output_coords(root->output_layout, wlr_output, &ox, &oy);
 
 	if (root->fullscreen_global) {
 		// Try fullscreen container
@@ -132,11 +139,6 @@ struct sway_node *node_at_coords(
 		return NULL;
 	}
 
-	if ((*surface = layer_surface_at(output,
-				&output->layers[ZWLR_LAYER_SHELL_V1_LAYER_OVERLAY],
-				ox, oy, sx, sy))) {
-		return NULL;
-	}
 	if (ws->fullscreen) {
 		// Try transient containers
 		for (int i = 0; i < ws->floating->length; ++i) {
@@ -496,7 +498,6 @@ static void handle_touch_down(struct wl_listener *listener, void *data) {
 		pointer_motion(cursor, event->time_msec, event->device, dx, dy, dx, dy);
 		dispatch_cursor_button(cursor, event->device, event->time_msec,
 				BTN_LEFT, WLR_BUTTON_PRESSED);
-		wlr_seat_pointer_notify_frame(wlr_seat);
 	}
 }
 
@@ -509,10 +510,9 @@ static void handle_touch_up(struct wl_listener *listener, void *data) {
 
 	if (cursor->simulating_pointer_from_touch) {
 		if (cursor->pointer_touch_id == cursor->seat->touch_id) {
-			cursor->simulating_pointer_from_touch = false;
+			cursor->pointer_touch_up = true;
 			dispatch_cursor_button(cursor, event->device, event->time_msec,
 					BTN_LEFT, WLR_BUTTON_RELEASED);
-			wlr_seat_pointer_notify_frame(wlr_seat);
 		}
 	} else {
 		wlr_seat_touch_notify_up(wlr_seat, event->time_msec, event->touch_id);
@@ -560,6 +560,24 @@ static void handle_touch_motion(struct wl_listener *listener, void *data) {
 	}
 }
 
+static void handle_touch_frame(struct wl_listener *listener, void *data) {
+	struct sway_cursor *cursor =
+		wl_container_of(listener, cursor, touch_frame);
+
+	struct wlr_seat *wlr_seat = cursor->seat->wlr_seat;
+
+	if (cursor->simulating_pointer_from_touch) {
+		wlr_seat_pointer_notify_frame(wlr_seat);
+
+		if (cursor->pointer_touch_up) {
+			cursor->pointer_touch_up = false;
+			cursor->simulating_pointer_from_touch = false;
+		}
+	} else {
+		wlr_seat_touch_notify_frame(wlr_seat);
+	}
+}
+
 static double apply_mapping_from_coord(double low, double high, double value) {
 	if (isnan(value)) {
 		return value;
@@ -909,6 +927,7 @@ static void handle_pointer_pinch_begin(struct wl_listener *listener, void *data)
 	struct sway_cursor *cursor = wl_container_of(
 			listener, cursor, pinch_begin);
 	struct wlr_event_pointer_pinch_begin *event = data;
+	cursor_handle_activity_from_device(cursor, event->device);
 	wlr_pointer_gestures_v1_send_pinch_begin(
 			cursor->pointer_gestures, cursor->seat->wlr_seat,
 			event->time_msec, event->fingers);
@@ -918,6 +937,7 @@ static void handle_pointer_pinch_update(struct wl_listener *listener, void *data
 	struct sway_cursor *cursor = wl_container_of(
 			listener, cursor, pinch_update);
 	struct wlr_event_pointer_pinch_update *event = data;
+	cursor_handle_activity_from_device(cursor, event->device);
 	wlr_pointer_gestures_v1_send_pinch_update(
 			cursor->pointer_gestures, cursor->seat->wlr_seat,
 			event->time_msec, event->dx, event->dy,
@@ -928,6 +948,7 @@ static void handle_pointer_pinch_end(struct wl_listener *listener, void *data) {
 	struct sway_cursor *cursor = wl_container_of(
 			listener, cursor, pinch_end);
 	struct wlr_event_pointer_pinch_end *event = data;
+	cursor_handle_activity_from_device(cursor, event->device);
 	wlr_pointer_gestures_v1_send_pinch_end(
 			cursor->pointer_gestures, cursor->seat->wlr_seat,
 			event->time_msec, event->cancelled);
@@ -937,6 +958,7 @@ static void handle_pointer_swipe_begin(struct wl_listener *listener, void *data)
 	struct sway_cursor *cursor = wl_container_of(
 			listener, cursor, swipe_begin);
 	struct wlr_event_pointer_swipe_begin *event = data;
+	cursor_handle_activity_from_device(cursor, event->device);
 	wlr_pointer_gestures_v1_send_swipe_begin(
 			cursor->pointer_gestures, cursor->seat->wlr_seat,
 			event->time_msec, event->fingers);
@@ -946,6 +968,7 @@ static void handle_pointer_swipe_update(struct wl_listener *listener, void *data
 	struct sway_cursor *cursor = wl_container_of(
 			listener, cursor, swipe_update);
 	struct wlr_event_pointer_swipe_update *event = data;
+	cursor_handle_activity_from_device(cursor, event->device);
 	wlr_pointer_gestures_v1_send_swipe_update(
 			cursor->pointer_gestures, cursor->seat->wlr_seat,
 			event->time_msec, event->dx, event->dy);
@@ -955,6 +978,7 @@ static void handle_pointer_swipe_end(struct wl_listener *listener, void *data) {
 	struct sway_cursor *cursor = wl_container_of(
 			listener, cursor, swipe_end);
 	struct wlr_event_pointer_swipe_end *event = data;
+	cursor_handle_activity_from_device(cursor, event->device);
 	wlr_pointer_gestures_v1_send_swipe_end(
 			cursor->pointer_gestures, cursor->seat->wlr_seat,
 			event->time_msec, event->cancelled);
@@ -1045,6 +1069,7 @@ void sway_cursor_destroy(struct sway_cursor *cursor) {
 	wl_list_remove(&cursor->touch_down.link);
 	wl_list_remove(&cursor->touch_up.link);
 	wl_list_remove(&cursor->touch_motion.link);
+	wl_list_remove(&cursor->touch_frame.link);
 	wl_list_remove(&cursor->tool_axis.link);
 	wl_list_remove(&cursor->tool_tip.link);
 	wl_list_remove(&cursor->tool_button.link);
@@ -1120,6 +1145,9 @@ struct sway_cursor *sway_cursor_create(struct sway_seat *seat) {
 		&cursor->touch_motion);
 	cursor->touch_motion.notify = handle_touch_motion;
 
+	wl_signal_add(&wlr_cursor->events.touch_frame, &cursor->touch_frame);
+	cursor->touch_frame.notify = handle_touch_frame;
+
 	wl_signal_add(&wlr_cursor->events.tablet_tool_axis,
 		&cursor->tool_axis);
 	cursor->tool_axis.notify = handle_tool_axis;
@@ -1327,7 +1355,7 @@ void handle_pointer_constraint(struct wl_listener *listener, void *data) {
 	wl_signal_add(&constraint->events.destroy, &sway_constraint->destroy);
 
 	struct sway_node *focus = seat_get_focus(seat);
-	if (focus && focus->type == N_CONTAINER && focus->sway_container->view) {
+	if (focus && node_is_view(focus)) {
 		struct wlr_surface *surface = focus->sway_container->view->surface;
 		if (surface == constraint->surface) {
 			sway_cursor_constrain(seat->cursor, constraint);
