@@ -9,10 +9,10 @@
 #include <wlr/types/wlr_keyboard_group.h>
 #include <xkbcommon/xkbcommon-names.h>
 #include "sway/commands.h"
-#include "sway/desktop/transaction.h"
 #include "sway/input/input-manager.h"
 #include "sway/input/keyboard.h"
 #include "sway/input/seat.h"
+#include "sway/input/cursor.h"
 #include "sway/ipc-server.h"
 #include "log.h"
 
@@ -124,7 +124,7 @@ static void state_add_key(struct sway_shortcut_state *state,
  * Update the shortcut model state in response to new input
  */
 static bool update_shortcut_state(struct sway_shortcut_state *state,
-		uint32_t keycode, enum wlr_key_state keystate, uint32_t new_key,
+		uint32_t keycode, enum wl_keyboard_key_state keystate, uint32_t new_key,
 		uint32_t raw_modifiers) {
 	bool last_key_was_a_modifier = raw_modifiers != state->last_raw_modifiers;
 	state->last_raw_modifiers = raw_modifiers;
@@ -134,7 +134,7 @@ static bool update_shortcut_state(struct sway_shortcut_state *state,
 		state_erase_key(state, state->last_keycode);
 	}
 
-	if (keystate == WLR_KEY_PRESSED) {
+	if (keystate == WL_KEYBOARD_KEY_STATE_PRESSED) {
 		// Add current key to set; there may be duplicates
 		state_add_key(state, keycode, new_key);
 		state->last_keycode = keycode;
@@ -348,7 +348,7 @@ struct key_info {
 };
 
 static void update_keyboard_state(struct sway_keyboard *keyboard,
-		uint32_t raw_keycode, enum wlr_key_state keystate,
+		uint32_t raw_keycode, enum wl_keyboard_key_state keystate,
 		struct key_info *keyinfo) {
 	// Identify new keycode, raw keysym(s), and translated keysym(s)
 	keyinfo->keycode = raw_keycode + 8;
@@ -378,6 +378,28 @@ static void update_keyboard_state(struct sway_keyboard *keyboard,
 	}
 }
 
+/**
+ * Get keyboard grab of the seat from sway_keyboard if we should forward events
+ * to it.
+ *
+ * Returns NULL if the keyboard is not grabbed by an input method,
+ * or if event is from virtual keyboard of the same client as grab.
+ * TODO: see swaywm/wlroots#2322
+ */
+static struct wlr_input_method_keyboard_grab_v2 *keyboard_get_im_grab(
+		struct sway_keyboard *keyboard) {
+	struct wlr_input_method_v2 *input_method = keyboard->seat_device->
+		sway_seat->im_relay.input_method;
+	struct wlr_virtual_keyboard_v1 *virtual_keyboard =
+		wlr_input_device_get_virtual_keyboard(keyboard->seat_device->input_device->wlr_device);
+	if (!input_method || !input_method->keyboard_grab || (virtual_keyboard &&
+				wl_resource_get_client(virtual_keyboard->resource) ==
+				wl_resource_get_client(input_method->keyboard_grab->resource))) {
+		return NULL;
+	}
+	return input_method->keyboard_grab;
+}
+
 static void handle_key_event(struct sway_keyboard *keyboard,
 		struct wlr_event_keyboard_key *event) {
 	struct sway_seat *seat = keyboard->seat_device->sway_seat;
@@ -392,6 +414,10 @@ static void handle_key_event(struct sway_keyboard *keyboard,
 		keyboard_shortcuts_inhibitor_get_for_focused_surface(seat);
 	bool shortcuts_inhibited = sway_inhibitor && sway_inhibitor->inhibitor->active;
 
+	if (event->state == WL_KEYBOARD_KEY_STATE_PRESSED) {
+		cursor_notify_key_press(seat->cursor);
+	}
+
 	// Identify new keycode, raw keysym(s), and translated keysym(s)
 	struct key_info keyinfo;
 	update_keyboard_state(keyboard, event->keycode, event->state, &keyinfo);
@@ -417,20 +443,20 @@ static void handle_key_event(struct sway_keyboard *keyboard,
 
 	// Execute stored release binding once no longer active
 	if (keyboard->held_binding && binding_released != keyboard->held_binding &&
-			event->state == WLR_KEY_RELEASED) {
+			event->state == WL_KEYBOARD_KEY_STATE_RELEASED) {
 		seat_execute_command(seat, keyboard->held_binding);
 		handled = true;
 	}
 	if (binding_released != keyboard->held_binding) {
 		keyboard->held_binding = NULL;
 	}
-	if (binding_released && event->state == WLR_KEY_PRESSED) {
+	if (binding_released && event->state == WL_KEYBOARD_KEY_STATE_PRESSED) {
 		keyboard->held_binding = binding_released;
 	}
 
 	// Identify and execute active pressed binding
 	struct sway_binding *binding = NULL;
-	if (event->state == WLR_KEY_PRESSED) {
+	if (event->state == WL_KEYBOARD_KEY_STATE_PRESSED) {
 		get_active_binding(&keyboard->state_keycodes,
 				config->current_mode->keycode_bindings, &binding,
 				keyinfo.code_modifiers, false, input_inhibited,
@@ -473,29 +499,53 @@ static void handle_key_event(struct sway_keyboard *keyboard,
 	}
 
 	// Compositor bindings
-	if (!handled && event->state == WLR_KEY_PRESSED) {
+	if (!handled && event->state == WL_KEYBOARD_KEY_STATE_PRESSED) {
 		handled = keyboard_execute_compositor_binding(
 				keyboard, keyinfo.translated_keysyms,
 				keyinfo.translated_modifiers, keyinfo.translated_keysyms_len);
 	}
-	if (!handled && event->state == WLR_KEY_PRESSED) {
+	if (!handled && event->state == WL_KEYBOARD_KEY_STATE_PRESSED) {
 		handled = keyboard_execute_compositor_binding(
 				keyboard, keyinfo.raw_keysyms, keyinfo.raw_modifiers,
 				keyinfo.raw_keysyms_len);
 	}
 
-	if (!handled || event->state == WLR_KEY_RELEASED) {
+	if (event->state == WL_KEYBOARD_KEY_STATE_RELEASED) {
+		// If the pressed event was sent to a client, also send the released
+		// event. In particular, don't send the released event to the IM grab.
 		bool pressed_sent = update_shortcut_state(
-				&keyboard->state_pressed_sent, event->keycode, event->state,
-				keyinfo.keycode, 0);
-		if (pressed_sent || event->state == WLR_KEY_PRESSED) {
+			&keyboard->state_pressed_sent, event->keycode,
+			event->state, keyinfo.keycode, 0);
+		if (pressed_sent) {
 			wlr_seat_set_keyboard(wlr_seat, wlr_device);
 			wlr_seat_keyboard_notify_key(wlr_seat, event->time_msec,
-					event->keycode, event->state);
+				event->keycode, event->state);
+			handled = true;
 		}
 	}
 
-	transaction_commit_dirty();
+	if (!handled) {
+		struct wlr_input_method_keyboard_grab_v2 *kb_grab = keyboard_get_im_grab(keyboard);
+
+		if (kb_grab) {
+			wlr_input_method_keyboard_grab_v2_set_keyboard(kb_grab,
+				wlr_device->keyboard);
+			wlr_input_method_keyboard_grab_v2_send_key(kb_grab,
+				event->time_msec, event->keycode, event->state);
+			handled = true;
+		}
+	}
+
+	if (!handled && event->state != WL_KEYBOARD_KEY_STATE_RELEASED) {
+		// If a released event failed pressed sent test, and not in sent to
+		// keyboard grab, it is still not handled. Don't handle released here.
+		update_shortcut_state(
+			&keyboard->state_pressed_sent, event->keycode, event->state,
+			keyinfo.keycode, 0);
+		wlr_seat_set_keyboard(wlr_seat, wlr_device);
+		wlr_seat_keyboard_notify_key(wlr_seat, event->time_msec,
+				event->keycode, event->state);
+	}
 
 	free(device_identifier);
 }
@@ -523,7 +573,7 @@ static void handle_keyboard_group_enter(struct wl_listener *listener,
 	uint32_t *keycode;
 	wl_array_for_each(keycode, keycodes) {
 		struct key_info keyinfo;
-		update_keyboard_state(keyboard, *keycode, WLR_KEY_PRESSED, &keyinfo);
+		update_keyboard_state(keyboard, *keycode, WL_KEYBOARD_KEY_STATE_PRESSED, &keyinfo);
 	}
 }
 
@@ -539,10 +589,10 @@ static void handle_keyboard_group_leave(struct wl_listener *listener,
 	uint32_t *keycode;
 	wl_array_for_each(keycode, keycodes) {
 		struct key_info keyinfo;
-		update_keyboard_state(keyboard, *keycode, WLR_KEY_RELEASED, &keyinfo);
+		update_keyboard_state(keyboard, *keycode, WL_KEYBOARD_KEY_STATE_RELEASED, &keyinfo);
 
 		pressed_sent |= update_shortcut_state(&keyboard->state_pressed_sent,
-				*keycode, WLR_KEY_RELEASED, keyinfo.keycode, 0);
+				*keycode, WL_KEYBOARD_KEY_STATE_RELEASED, keyinfo.keycode, 0);
 	}
 
 	if (!pressed_sent) {
@@ -582,7 +632,6 @@ static int handle_keyboard_repeat(void *data) {
 
 		seat_execute_command(keyboard->seat_device->sway_seat,
 				keyboard->repeat_binding);
-		transaction_commit_dirty();
 	}
 	return 0;
 }
@@ -612,19 +661,30 @@ static void handle_modifier_event(struct sway_keyboard *keyboard) {
 	struct wlr_input_device *wlr_device =
 		keyboard->seat_device->input_device->wlr_device;
 	if (!wlr_device->keyboard->group) {
-		struct wlr_seat *wlr_seat = keyboard->seat_device->sway_seat->wlr_seat;
-		wlr_seat_set_keyboard(wlr_seat, wlr_device);
-		wlr_seat_keyboard_notify_modifiers(wlr_seat,
-				&wlr_device->keyboard->modifiers);
+		struct wlr_input_method_keyboard_grab_v2 *kb_grab = keyboard_get_im_grab(keyboard);
+
+		if (kb_grab) {
+			wlr_input_method_keyboard_grab_v2_set_keyboard(kb_grab,
+					wlr_device->keyboard);
+			wlr_input_method_keyboard_grab_v2_send_modifiers(kb_grab,
+					&wlr_device->keyboard->modifiers);
+		} else {
+			struct wlr_seat *wlr_seat = keyboard->seat_device->sway_seat->wlr_seat;
+			wlr_seat_set_keyboard(wlr_seat, wlr_device);
+			wlr_seat_keyboard_notify_modifiers(wlr_seat,
+					&wlr_device->keyboard->modifiers);
+		}
 
 		uint32_t modifiers = wlr_keyboard_get_modifiers(wlr_device->keyboard);
 		determine_bar_visibility(modifiers);
 	}
 
-	if (wlr_device->keyboard->modifiers.group != keyboard->effective_layout &&
-			!wlr_keyboard_group_from_wlr_keyboard(wlr_device->keyboard)) {
+	if (wlr_device->keyboard->modifiers.group != keyboard->effective_layout) {
 		keyboard->effective_layout = wlr_device->keyboard->modifiers.group;
-		ipc_event_input("xkb_layout", keyboard->seat_device->input_device);
+
+		if (!wlr_keyboard_group_from_wlr_keyboard(wlr_device->keyboard)) {
+			ipc_event_input("xkb_layout", keyboard->seat_device->input_device);
+		}
 	}
 }
 
