@@ -51,6 +51,16 @@ static void seat_device_destroy(struct sway_seat_device *seat_device) {
 static void seat_node_destroy(struct sway_seat_node *seat_node) {
 	wl_list_remove(&seat_node->destroy.link);
 	wl_list_remove(&seat_node->link);
+
+	/*
+	 * This is the only time we remove items from the focus stack without
+	 * immediately re-adding them. If we just removed the last thing,
+	 * mark that nothing has focus anymore.
+	 */
+	if (wl_list_empty(&seat_node->seat->focus_stack)) {
+		seat_node->seat->has_focus = false;
+	}
+
 	free(seat_node);
 }
 
@@ -210,14 +220,13 @@ void seat_for_each_node(struct sway_seat *seat,
 
 struct sway_container *seat_get_focus_inactive_view(struct sway_seat *seat,
 		struct sway_node *ancestor) {
-	if (ancestor->type == N_CONTAINER && ancestor->sway_container->view) {
+	if (node_is_view(ancestor)) {
 		return ancestor->sway_container;
 	}
 	struct sway_seat_node *current;
 	wl_list_for_each(current, &seat->focus_stack, link) {
 		struct sway_node *node = current->node;
-		if (node->type == N_CONTAINER && node->sway_container->view &&
-				node_has_ancestor(node, ancestor)) {
+		if (node_is_view(node) && node_has_ancestor(node, ancestor)) {
 			return node->sway_container;
 		}
 	}
@@ -1416,9 +1425,8 @@ struct sway_node *seat_get_focus(struct sway_seat *seat) {
 	if (!seat->has_focus) {
 		return NULL;
 	}
-	if (wl_list_empty(&seat->focus_stack)) {
-		return NULL;
-	}
+	sway_assert(!wl_list_empty(&seat->focus_stack),
+			"focus_stack is empty, but has_focus is true");
 	struct sway_seat_node *current =
 		wl_container_of(seat->focus_stack.next, current, link);
 	return current->node;
