@@ -1,7 +1,11 @@
+#include <ctype.h>
+#include <float.h>
 #include <json.h>
 #include <libevdev/libevdev.h>
 #include <stdio.h>
-#include <ctype.h>
+#include <wlr/backend/libinput.h>
+#include <wlr/types/wlr_output.h>
+#include <xkbcommon/xkbcommon.h>
 #include "config.h"
 #include "log.h"
 #include "sway/config.h"
@@ -13,10 +17,6 @@
 #include "sway/input/input-manager.h"
 #include "sway/input/cursor.h"
 #include "sway/input/seat.h"
-#include <wlr/backend/libinput.h>
-#include <wlr/types/wlr_box.h>
-#include <wlr/types/wlr_output.h>
-#include <xkbcommon/xkbcommon.h>
 #include "wlr-layer-shell-unstable-v1-protocol.h"
 #include "sway/desktop/idle_inhibit_v1.h"
 
@@ -1002,6 +1002,17 @@ json_object *ipc_json_describe_input(struct sway_input_device *device) {
 		}
 	}
 
+	if (device->wlr_device->type == WLR_INPUT_DEVICE_POINTER) {
+		struct input_config *ic = input_device_get_config(device);
+		float scroll_factor = 1.0f;
+		if (ic != NULL && !isnan(ic->scroll_factor) && 
+				ic->scroll_factor != FLT_MIN) {
+			scroll_factor = ic->scroll_factor;
+		}
+		json_object_object_add(object, "scroll_factor", 
+				json_object_new_double(scroll_factor));
+	}
+
 	if (wlr_input_device_is_libinput(device->wlr_device)) {
 		struct libinput_device *libinput_dev;
 		libinput_dev = wlr_libinput_get_device_handle(device->wlr_device);
@@ -1115,7 +1126,9 @@ json_object *ipc_json_describe_bar_config(struct bar_config *bar) {
 	json_object_object_add(json, "verbose",
 			json_object_new_boolean(bar->verbose));
 	json_object_object_add(json, "pango_markup",
-			json_object_new_boolean(bar->pango_markup));
+			json_object_new_boolean(bar->pango_markup == PANGO_MARKUP_DEFAULT
+											? config->pango_markup
+											: bar->pango_markup));
 
 	json_object *colors = json_object_new_object();
 	json_object_object_add(colors, "background",
