@@ -687,7 +687,7 @@ void ipc_client_handle_command(struct ipc_client *client, uint32_t payload_lengt
 		}
 		struct sway_output *output;
 		wl_list_for_each(output, &root->all_outputs, link) {
-			if (!output->enabled && output != root->noop_output) {
+			if (!output->enabled && output != root->fallback_output) {
 				json_object_array_add(outputs,
 						ipc_json_describe_disabled_output(output));
 			}
