@@ -6,6 +6,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/resource.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <sys/wait.h>
@@ -27,6 +28,7 @@
 
 static bool terminate_request = false;
 static int exit_value = 0;
+static struct rlimit original_nofile_rlimit = {0};
 struct sway_server server = {0};
 struct sway_debug debug = {0};
 
@@ -47,43 +49,6 @@ void sig_handler(int signal) {
 	sway_terminate(EXIT_SUCCESS);
 }
 
-void detect_raspi(void) {
-	bool raspi = false;
-	FILE *f = fopen("/sys/firmware/devicetree/base/model", "r");
-	if (!f) {
-		return;
-	}
-	char *line = NULL;
-	size_t line_size = 0;
-	while (getline(&line, &line_size, f) != -1) {
-		if (strstr(line, "Raspberry Pi")) {
-			raspi = true;
-			break;
-		}
-	}
-	fclose(f);
-	FILE *g = fopen("/proc/modules", "r");
-	if (!g) {
-		free(line);
-		return;
-	}
-	bool vc4 = false;
-	while (getline(&line, &line_size, g) != -1) {
-		if (strstr(line, "vc4")) {
-			vc4 = true;
-			break;
-		}
-	}
-	free(line);
-	fclose(g);
-	if (!vc4 && raspi) {
-		fprintf(stderr, "\x1B[1;31mWarning: You have a "
-				"Raspberry Pi, but the vc4 Module is "
-				"not loaded! Set 'dtoverlay=vc4-kms-v3d'"
-				"in /boot/config.txt and reboot.\x1B[0m\n");
-	}
-}
-
 void detect_proprietary(int allow_unsupported_gpu) {
 	FILE *f = fopen("/proc/modules", "r");
 	if (!f) {
@@ -100,7 +65,7 @@ void detect_proprietary(int allow_unsupported_gpu) {
 				sway_log(SWAY_ERROR,
 					"Proprietary Nvidia drivers are NOT supported. "
 					"Use Nouveau. To launch sway anyway, launch with "
-					"--my-next-gpu-wont-be-nvidia and DO NOT report issues.");
+					"--unsupported-gpu and DO NOT report issues.");
 				exit(EXIT_FAILURE);
 			}
 			break;
@@ -188,6 +153,9 @@ static void log_kernel(void) {
 
 static bool drop_permissions(void) {
 	if (getuid() != geteuid() || getgid() != getegid()) {
+		sway_log(SWAY_ERROR, "!!! DEPRECATION WARNING: "
+			"SUID privilege drop will be removed in a future release, please migrate to seatd-launch");
+
 		// Set the gid and uid in the correct order.
 		if (setgid(getgid()) != 0) {
 			sway_log(SWAY_ERROR, "Unable to drop root group, refusing to start");
@@ -206,6 +174,33 @@ static bool drop_permissions(void) {
 	return true;
 }
 
+static void increase_nofile_limit(void) {
+	if (getrlimit(RLIMIT_NOFILE, &original_nofile_rlimit) != 0) {
+		sway_log_errno(SWAY_ERROR, "Failed to bump max open files limit: "
+			"getrlimit(NOFILE) failed");
+		return;
+	}
+
+	struct rlimit new_rlimit = original_nofile_rlimit;
+	new_rlimit.rlim_cur = new_rlimit.rlim_max;
+	if (setrlimit(RLIMIT_NOFILE, &new_rlimit) != 0) {
+		sway_log_errno(SWAY_ERROR, "Failed to bump max open files limit: "
+			"setrlimit(NOFILE) failed");
+		sway_log(SWAY_INFO, "Running with %d max open files",
+			(int)original_nofile_rlimit.rlim_cur);
+	}
+}
+
+void restore_nofile_limit(void) {
+	if (original_nofile_rlimit.rlim_cur == 0) {
+		return;
+	}
+	if (setrlimit(RLIMIT_NOFILE, &original_nofile_rlimit) != 0) {
+		sway_log_errno(SWAY_ERROR, "Failed to restore max open files limit: "
+			"setrlimit(NOFILE) failed");
+	}
+}
+
 void enable_debug_flag(const char *flag) {
 	if (strcmp(flag, "damage=highlight") == 0) {
 		debug.damage = DAMAGE_HIGHLIGHT;
@@ -219,6 +214,8 @@ void enable_debug_flag(const char *flag) {
 		debug.txn_timings = true;
 	} else if (strncmp(flag, "txn-timeout=", 12) == 0) {
 		server.txn_timeout_ms = atoi(&flag[12]);
+	} else if (strcmp(flag, "noscanout") == 0) {
+		debug.noscanout = true;
 	} else {
 		sway_log(SWAY_ERROR, "Unknown debug flag: %s", flag);
 	}
@@ -255,7 +252,6 @@ int main(int argc, char **argv) {
 		{"verbose", no_argument, NULL, 'V'},
 		{"get-socketpath", no_argument, NULL, 'p'},
 		{"unsupported-gpu", no_argument, NULL, 'u'},
-		{"my-next-gpu-wont-be-nvidia", no_argument, NULL, 'u'},
 		{0, 0, 0, 0}
 	};
 
@@ -349,8 +345,6 @@ int main(int argc, char **argv) {
 	log_kernel();
 	log_distro();
 	log_env();
-	detect_proprietary(allow_unsupported_gpu);
-	detect_raspi();
 
 	if (optind < argc) { // Behave as IPC client
 		if (optind != 1) {
@@ -377,6 +371,8 @@ int main(int argc, char **argv) {
 		return 0;
 	}
 
+	detect_proprietary(allow_unsupported_gpu);
+
 	if (!server_privileged_prepare(&server)) {
 		return 1;
 	}
@@ -386,6 +382,8 @@ int main(int argc, char **argv) {
 		exit(EXIT_FAILURE);
 	}
 
+	increase_nofile_limit();
+
 	// handle SIGTERM signals
 	signal(SIGTERM, sig_handler);
 	signal(SIGINT, sig_handler);
